'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var meow = _interopDefault(require('meow'));
var detectNewline = _interopDefault(require('detect-newline'));
var boxen = _interopDefault(require('boxen'));
var chalk = _interopDefault(require('chalk'));
var SimpleGit = _interopDefault(require('simple-git'));
var log = _interopDefault(require('git-raw-commits'));

async function showNotifications(messageStream, flags) {
  let detectedNewline; // for each commit message

  for await (const chunk of messageStream) {
    // read message in stream
    const message = chunk.toString();
    detectedNewline = detectedNewline || detectNewline(message); // for each paragraph in message

    const newline = detectedNewline || '\n';
    const paragraphs = message.split(`${newline}${newline}`);

    for (const paragraph of paragraphs) {
      const notifications = paragraph.split(flags.prefix).slice(1); // for each notification on this paragraph

      notifications.forEach(notification => {
        // display notification
        notify(notification.trim(), flags);
      });
    }
  }
}

function notify(message, flags) {
  const chalkPreset = chalk[flags.color];
  const simpleColor = chalkPreset || chalk.hex(flags.color);
  const borderColor = chalkPreset ? flags.color : flags.color.startsWith('#') ? flags.color : `#${flags.color}`;
  const display = flags.simple ? simpleColor(message) : boxen(message, {
    padding: 1,
    margin: 1,
    borderStyle: 'doubleSingle',
    borderColor,
    float: 'center',
    align: 'center'
  }); // notify to console

  console.log(display);
}

const git = /*#__PURE__*/SimpleGit();
function getLastRef() {
  return git.raw('reflog', '--pretty=format:"%h"', '--no-patch', '-1', 'HEAD@{1}').then(value => value.replace(/"/g, ''));
}
async function getLogStream(lifecycle, args) {
  switch (lifecycle) {
    case 'merge':
      return log({
        from: await getLastRef()
      });

    case 'rewrite':
      // perhaps not the most accurate method, PRs welcome
      return log({
        from: 'origin',
        to: 'HEAD'
      });

    case 'checkout':
      const [from, to] = args;
      return log({
        from,
        to
      });

    case 'since':
      return log({
        from: args[0]
      });

    default:
      throw new Error('[git-notify] unsupported git hook: ' + lifecycle);
  }
}

// import commits from 'git-raw-commits';

async function hook(lifecycle, args, flags) {
  // get all commit messages for the relevant revision range
  // depending on which git hook / command was executed
  const logs = await getLogStream(lifecycle, args); // stream through logs and print any found notifications

  showNotifications(logs, flags);
}

const cli = /*#__PURE__*/meow(`
	Usage
	  $ git-notify <method> [options] $GIT_PARAMS

  Methods
    since <commit>  show all notifications since commit
    merge           run on git pull/merge
    rewrite         run on git rebase
    checkout        run on git checkout/switch

	Options
    --prefix, -p    prefix to  look for in commit messages (default: "git-notify:")
    --simple, -s    show a plain, unboxed notification
    --color, -c     color of displayed notification

	Examples
    $ git-notify since HEAD~5
    $ git-notify checkout $GIT_PARAMS --prefix "\@everyone:"
    $ git-notify merge $GIT_PARAMS --simple --color "\#ff6f6f"
`, {
  flags: {
    toast: {
      type: 'boolean',
      alias: 't'
    },
    simple: {
      type: 'boolean',
      alias: 's'
    },
    prefix: {
      type: 'string',
      alias: 'p',
      default: 'git-notify:'
    },
    color: {
      type: 'string',
      alias: 'c',
      default: 'cyan',
      description: 'hex value like #ff0000, or one of: black, red, green, yellow, blue, magenta, cyan, white, gray'
    }
  }
});
hook(cli.input[0], cli.input.slice(1), cli.flags);
//# sourceMappingURL=git-notify.cjs.development.js.map
